@@ -16,18 +16,9 @@ class GraphImporterInterface(ABC):
     def _import(self, graph) -> Tuple[List[Dict], List[Dict], bool]:
         """import graph"""
 
+    def isInstance(self, graph):
+        return self._type is not None and isinstance(graph, self._type)
+
     def __call__(self, graph):
-        assert isinstance(graph, self._type)
+        assert self.isInstance(graph)
         return self._import(graph)
-
-
-class GraphImporterFactoryInterface(ABC):
-    """Common factory interface for graph importers.
-
-    Provides only one graph importer and
-    does not maintain any of the instances it creates.
-    """
-
-    @abstractmethod
-    def get_graph_importer(self) -> GraphImporterInterface:
-        """return a new graph importer instance"""
