@@ -26,7 +26,8 @@ CONTEXT_PANE_MAPPING: list
     it makes it possible (and easier) to check user input for correctness.
 """
 
-from typing import Any, Dict as TDict, List as TList, Optional
+from typing import Any, Dict as TDict, List as TList, Optional, Union
+import inspect
 
 from IPython.display import SVG
 from ipywidgets import DOMWidget, Layout
@@ -273,8 +274,8 @@ class GraphWidget(DOMWidget):
 
         Returns
         -------
-        node_label_mapping: callable
-            A function that produces node labels.
+        node_label_mapping: callable | str
+            A function that produces node labels or the name of the property to use for the label binding.
 
         """
         return self._get_attribute_by_name('_node_label_mapping', 'default')
@@ -284,9 +285,9 @@ class GraphWidget(DOMWidget):
 
         Parameters
         ----------
-        node_label_mapping: callable
-            A function that produces node labels.
-            It should have the same signature as `default_node_label_mapping`
+        node_label_mapping: callable | str
+            A function that produces node labels or the name of the property to use for the label binding.
+            The function should have the same signature as `default_node_label_mapping`
             e.g. take in an index and node dictionary and return a string.
 
         Example
@@ -328,8 +329,8 @@ class GraphWidget(DOMWidget):
 
         Returns
         -------
-        edge_label_mapping: callable
-            A function that produces edge labels.
+        edge_label_mapping: callable | str
+            A function that produces edge labels or the name of the property to use for the label binding.
 
         """
         return self._get_attribute_by_name('_edge_label_mapping', 'default')
@@ -339,9 +340,9 @@ class GraphWidget(DOMWidget):
 
         Parameters
         ----------
-        edge_label_mapping: callable
-            A function that produces edge labels.
-            It should have the same signature as `default_edge_label_mapping`
+        edge_label_mapping: callable | str
+            A function that produces edge labels or the name of the property to use for the label binding.
+            The function should have the same signature as `default_edge_label_mapping`
             e.g. take in an index and edge dictionary and return a string.
 
         Example
@@ -383,8 +384,8 @@ class GraphWidget(DOMWidget):
 
         Returns
         -------
-        node_property_mapping: callable
-            A function that produces node properties.
+        node_property_mapping: callable | str
+            A function that produces node properties or the name of the property to use for the property binding.
 
         """
         return self._get_attribute_by_name('_node_property_mapping', 'default')
@@ -394,9 +395,9 @@ class GraphWidget(DOMWidget):
 
         Parameters
         ----------
-        node_property_mapping: callable
-            A function that produces node properties.
-            It should have the same signature as `default_node_property_mapping`
+        node_property_mapping: callable | str
+            A function that produces node properties or the name of the property to use for the property binding.
+            The function should have the same signature as `default_node_property_mapping`
             e.g. take in an index and node dictionary and return a dictionary.
 
         Notes
@@ -442,8 +443,8 @@ class GraphWidget(DOMWidget):
 
         Returns
         -------
-        edge_property_mapping: callable
-            A function that produces edge properties.
+        edge_property_mapping: callable | str
+            A function that produces edge properties or the name of the property to use for the property binding.
 
         """
         return self._get_attribute_by_name('_edge_property_mapping', 'default')
@@ -453,9 +454,9 @@ class GraphWidget(DOMWidget):
 
         Parameters
         ----------
-        edge_property_mapping: callable
-            A function that produces edge properties.
-            It should have the same signature as `default_edge_property_mapping`
+        edge_property_mapping: callable | str
+            A function that produces edge properties or the name of the property to use for the property binding.
+            The funtion should have the same signature as `default_edge_property_mapping`
             e.g. take in an index and edge dictionary and return a dictionary.
 
         Notes
@@ -501,8 +502,8 @@ class GraphWidget(DOMWidget):
 
         Returns
         -------
-        node_color_mapping: callable
-            A function that produces node colors.
+        node_color_mapping: callable | str
+            A function that produces node colors or the name of the property to use for the color binding.
 
         """
         return self._get_attribute_by_name('_node_color_mapping', 'default')
@@ -512,9 +513,9 @@ class GraphWidget(DOMWidget):
 
         Parameters
         ----------
-        node_color_mapping: callable
-            A function that produces node colors.
-            It should have the same signature as `default_node_color_mapping`
+        node_color_mapping: callable | str
+            A function that produces node colors or the name of the property to use for the color binding.
+            The function should have the same signature as `default_node_color_mapping`
             e.g. take in an index and node dictionary and return a string.
 
         Example
@@ -556,8 +557,8 @@ class GraphWidget(DOMWidget):
 
         Returns
         -------
-        edge_color_mapping: callable
-            A function that produces edge colors.
+        edge_color_mapping: callable | str
+            A function that produces edge colors or the name of the property to use for the color binding.
 
         """
         return self._get_attribute_by_name('_edge_color_mapping', 'default')
@@ -567,9 +568,9 @@ class GraphWidget(DOMWidget):
 
         Parameters
         ----------
-        edge_color_mapping: callable
-            A function that produces edge colors.
-            It should have the same signature as `default_edge_color_mapping`
+        edge_color_mapping: callable | str
+            A function that produces edge colors or the name of the property to use for the color binding.
+            The function should have the same signature as `default_edge_color_mapping`
             e.g. take in an index and edge dictionary and return a string.
 
         Example
@@ -602,6 +603,61 @@ class GraphWidget(DOMWidget):
 
     edge_color_mapping = property(get_edge_color_mapping, set_edge_color_mapping, del_edge_color_mapping)
 
+    def get_node_styles_mapping(self):
+        """Getter for the node styles mapping mapping property.
+
+        Notes
+        -----
+        If no mapping is explicitly set, `default_node_styles_mapping` is returned.
+
+        Returns
+        -------
+        node_styles_mapping: callable | str
+            A function that produces node styles or the name of the property to use for the style object binding.
+
+        """
+        return self._get_attribute_by_name('_node_styles_mapping', 'default')
+
+    def set_node_styles_mapping(self, node_styles_mapping):
+        """Setter for the node styles mapping mapping property.
+
+        Parameters
+        ----------
+        node_styles_mapping: callable | str
+            A function that produces node styles or the name of the property to use for the style object binding.
+            The function should have the same signature as `default_node_styles_mapping`
+            e.g. take in an index and node dictionary and return a typing.Dict.
+
+        Example
+        -------
+        .. code::
+
+           from yfiles_jupyter_graphs import GraphWidget
+           w = GraphWidget()
+           def custom_node_styles_mapping(index: int, node: dict):
+           ...
+           w.set_node_styles_mapping(custom_node_styles_mapping)
+
+        Returns
+        -------
+
+        """
+        # noinspection PyAttributeOutsideInit
+        self._node_styles_mapping = node_styles_mapping
+
+    def del_node_styles_mapping(self):
+        """Deleter for the node styles mapping mapping property.
+
+        Remove a custom node styles mapping.
+
+        Returns
+        -------
+
+        """
+        del self._node_styles_mapping
+
+    node_styles_mapping = property(get_node_styles_mapping, set_node_styles_mapping, del_node_styles_mapping)
+
     def get_node_scale_factor_mapping(self):
         """Getter for the node scale factor mapping property.
 
@@ -611,8 +667,8 @@ class GraphWidget(DOMWidget):
 
         Returns
         -------
-        node_scale_factor_mapping: callable
-            A function that produces node scale factor.
+        node_scale_factor_mapping: callable | str
+            A function that produces node scale factor or the name of the property to use for the scale binding.
 
         """
         return self._get_attribute_by_name('_node_scale_factor_mapping', 'default')
@@ -622,9 +678,9 @@ class GraphWidget(DOMWidget):
 
         Parameters
         ----------
-        node_scale_factor_mapping: callable
-            A function that produces node scale factors.
-            It should have the same signature as `default_node_scale_factor_mapping`
+        node_scale_factor_mapping: callable | str
+            A function that produces node scale factors or the name of the property to use for the scale binding.
+            The function should have the same signature as `default_node_scale_factor_mapping`
             e.g. take in an index and node dictionary and return a positive float.
 
         Example
@@ -667,8 +723,8 @@ class GraphWidget(DOMWidget):
 
         Returns
         -------
-        edge_thickness_factor_mapping: callable
-            A function that produces edge thickness factors.
+        edge_thickness_factor_mapping: callable | str
+            A function that produces edge thickness factors or the name of the property to use for the thickness binding.
 
         """
         return self._get_attribute_by_name('_edge_thickness_factor_mapping', 'default')
@@ -678,9 +734,9 @@ class GraphWidget(DOMWidget):
 
         Parameters
         ----------
-        edge_thickness_factor_mapping: callable
-            A function that produces edge thickness factors.
-            It should have the same signature as `default_edge_thickness_factor_mapping`
+        edge_thickness_factor_mapping: callable | str
+            A function that produces edge thickness factors or the name of the property to use for the thickness binding.
+            The function should have the same signature as `default_edge_thickness_factor_mapping`
             e.g. take in an index and edge dictionary and return a positive float.
 
         Example
@@ -723,8 +779,8 @@ class GraphWidget(DOMWidget):
 
         Returns
         -------
-        node_type_mapping: callable
-            A function that produces node types.
+        node_type_mapping: callable | str
+            A function that produces node types or the name of the property to use for the type binding.
 
         """
         return self._get_attribute_by_name('_node_type_mapping', 'default')
@@ -734,9 +790,9 @@ class GraphWidget(DOMWidget):
 
         Parameters
         ----------
-        node_type_mapping: callable
-            A function that produces node types.
-            It should have the same signature as `default_node_position_mapping`
+        node_type_mapping: callable | str
+            A function that produces node types or the name of the property to use for the type binding.
+            The function should have the same signature as `default_node_position_mapping`
             e.g. take in an index and node dictionary and return a bool/int/float or str value.
 
         Notes
@@ -786,8 +842,8 @@ class GraphWidget(DOMWidget):
 
         Returns
         -------
-        node_position_mapping: callable
-            A function that produces node positions.
+        node_position_mapping: callable | str
+            A function that produces node positions or the name of the property to use for position binding.
 
         """
         return self._get_attribute_by_name('_node_position_mapping', 'default')
@@ -797,9 +853,9 @@ class GraphWidget(DOMWidget):
 
         Parameters
         ----------
-        node_position_mapping: callable
-            A function that produces node positions.
-            It should have the same signature as `default_node_position_mapping`
+        node_position_mapping: callable | str
+            A function that produces node positions or the name of the property to use for the position binding.
+            The function should have the same signature as `default_node_position_mapping`
             e.g. take in an index and node dictionary and return a float 2-tuple.
 
         Notes
@@ -846,8 +902,8 @@ class GraphWidget(DOMWidget):
 
         Returns
         -------
-        directed_mapping: callable
-            A function that produces edge directions.
+        directed_mapping: callable | str
+            A function that produces edge directions or the name of the property to use for the direction binding.
 
         """
         return self._get_attribute_by_name('_directed_mapping', 'default')
@@ -857,9 +913,9 @@ class GraphWidget(DOMWidget):
 
         Parameters
         ----------
-        directed_mapping: callable
-            A function that produces edge directions.
-            It should have the same signature as `default_directed_mapping`
+        directed_mapping: callable | str
+            A function that produces edge directions or the name of the property to use for the direction binding.
+            The function should have the same signature as `default_directed_mapping`
             e.g. take in an index and edge dictionary and return a boolean value.
 
         Example
@@ -926,7 +982,6 @@ class GraphWidget(DOMWidget):
 
         - properties["label"]
         - properties["yf_label"]
-        - index
 
         Parameters
         ----------
@@ -955,7 +1010,7 @@ class GraphWidget(DOMWidget):
 
         """
         properties = element.get('properties', {})
-        return str(properties.get('label', properties.get('yf_label', index)))
+        return str(properties.get('label', properties.get('yf_label', '')))
 
     @staticmethod
     def default_node_label_mapping(index: int, node: TDict):
@@ -1054,6 +1109,53 @@ class GraphWidget(DOMWidget):
         """
         return '#17bebb'
 
+    # noinspection PyUnusedLocal
+    @staticmethod
+    def default_node_styles_mapping(index: int, node: TDict):
+        """The default styles mapping for nodes.
+
+        Parameters
+        ----------
+        index: int
+        node: typing.Dict
+
+        Notes
+        -----
+        This is the default value for the `node_styles_mapping` property.
+        Can be 'overwritten' by setting the property
+        with a function of the same signature.
+
+        Example
+        -------
+        .. code::
+
+           from yfiles_jupyter_graphs import GraphWidget
+           w = GraphWidget()
+           def custom_node_styles_mapping(index: int, node: typing.Dict):
+           ...
+           w.set_node_styles_mapping(custom_node_styles_mapping)
+
+        Returns
+        -------
+        
+        styles: typing.Dict
+            can contain the following key-value-pairs:
+                "color": str
+                    css color value
+                "shape": str
+                    possible values: 'ellipse', 'hexagon', 'hexagon2', 'octagon', 'pill', 'rectangle', 'round-rectangle' or 'triangle'
+                "image": str
+                    url or data URL of the image
+
+        References
+        ----------
+        `css color value <https://developer.mozilla.org/en-US/docs/Web/CSS/color_value>`_
+
+        `Data URL <https://developer.mozilla.org/en-US/docs/Web/HTTP/Basics_of_HTTP/Data_URLs>`_
+
+        """
+        return {}
+
     # noinspection PyUnusedLocal
     @staticmethod
     def default_edge_color_mapping(index: int, edge: TDict):
@@ -1283,7 +1385,7 @@ class GraphWidget(DOMWidget):
         Parameters
         ----------
         attribute_name: str
-        fallback_to: bool
+        fallback_to: typing.Optional[str]
 
         Returns
         -------
@@ -1299,7 +1401,7 @@ class GraphWidget(DOMWidget):
 
     @staticmethod
     def _get_wrapped_mapping_function(
-            function: callable,
+            function: Union[callable, str],
             key: str,
             strict: bool = True,
             remove_keys: Optional[TList[str]] = None
@@ -1311,8 +1413,8 @@ class GraphWidget(DOMWidget):
 
         Parameters
         ----------
-        function: callable
-            function to be wrapped
+        function: callable | str
+            function to be wrapped or the property key whose value should be returned
         key: str
             element (dict) key that should be used
             to save the return value of the function
@@ -1328,9 +1430,22 @@ class GraphWidget(DOMWidget):
 
         """
 
+        # helper function
+        def _get_function_value(function: Union[callable, str], index: int, element: TDict):
+            if isinstance(function, str):
+                return element.get('properties',{}).get(function)
+            
+            paramlen = len(inspect.signature(function).parameters)
+            if paramlen == 0:
+                return function()
+            elif paramlen == 1:
+                return function(index)
+            else:
+                return function(index, element)
+
         def wrapped(index: int, element: TDict):
             """wrap mapping function"""
-            value = function(index, element)
+            value = _get_function_value(function, index, element)
             assert not strict or key not in element.keys()
             element[key] = value
             if remove_keys:
@@ -1356,6 +1471,7 @@ class GraphWidget(DOMWidget):
             '_node_label_mapping': {'key': 'label', 'strict': False},
             '_node_property_mapping': {'key': 'properties', 'strict': False},
             '_node_color_mapping': {'key': 'color', 'strict': False},
+            '_node_styles_mapping': {'key': 'styles', 'strict': False},
             '_node_scale_factor_mapping': {'key': 'scale_factor', 'strict': False},
             '_node_type_mapping': {'key': 'type', 'strict': False},
             '_node_position_mapping': {'key': 'position', 'strict': False}
